@@ -1,19 +1,7 @@
 
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
 
 
 """
@@ -22,7 +10,9 @@ Test cases for defer module.
 
 from __future__ import nested_scopes
 
-from twisted.trial import unittest
+import random
+
+from twisted.trial import unittest, util
 from twisted.internet import reactor, defer
 from twisted.python import failure, log
 
@@ -141,6 +131,9 @@ class DeferredTestCase(unittest.TestCase):
         dl.addDeferred(defr2)
         defr2.callback('b')
         self.failUnlessEqual(result, [('a', 0)])
+    testEmptyDeferredList = util.suppressWarnings(testEmptyDeferredList,
+                                                  ('DeferredList.addDeferred',
+                                                   DeprecationWarning))
 
     def testDeferredListFireOnOneError(self):
         defr1 = defer.Deferred()
@@ -335,9 +328,9 @@ class DeferredTestCase(unittest.TestCase):
 
 class AlreadyCalledTestCase(unittest.TestCase):
     def setUp(self):
-        defer.Deferred.debug = True
+        defer.setDebugging(True)
     def tearDown(self):
-        defer.Deferred.debug = False
+        defer.setDebugging(False)
 
     def _callback(self, *args, **kw):
         pass
@@ -445,7 +438,7 @@ class AlreadyCalledTestCase(unittest.TestCase):
             self.fail("second errback failed to raise AlreadyCalledError")
 
     def testNoDebugging(self):
-        defer.Deferred.debug = False
+        defer.setDebugging(False)
         d = defer.Deferred()
         d.addCallbacks(self._callback, self._errback)
         self._call_1(d)
@@ -495,3 +488,129 @@ class DeferredTestCaseII(unittest.TestCase):
 
     def tearDown(self):
         self.failUnless(self.callbackRan, "Callback was never run.")
+
+class OtherPrimitives(unittest.TestCase):
+    def _incr(self, result):
+        self.counter += 1
+
+    def setUp(self):
+        self.counter = 0
+
+    def testLock(self):
+        lock = defer.DeferredLock()
+        lock.acquire().addCallback(self._incr)
+        self.failUnless(lock.locked)
+        self.assertEquals(self.counter, 1)
+
+        lock.acquire().addCallback(self._incr)
+        self.failUnless(lock.locked)
+        self.assertEquals(self.counter, 1)
+
+        lock.release()
+        self.failUnless(lock.locked)
+        self.assertEquals(self.counter, 2)
+
+        lock.release()
+        self.failIf(lock.locked)
+        self.assertEquals(self.counter, 2)
+
+        self.assertRaises(TypeError, lock.run)
+
+        firstUnique = object()
+        secondUnique = object()
+
+        controlDeferred = defer.Deferred()
+        def helper(self, b):
+            self.b = b
+            return controlDeferred
+
+        resultDeferred = lock.run(helper, self=self, b=firstUnique)
+        self.failUnless(lock.locked)
+        self.assertEquals(self.b, firstUnique)
+
+        resultDeferred.addCallback(lambda x: setattr(self, 'result', x))
+
+        lock.acquire().addCallback(self._incr)
+        self.failUnless(lock.locked)
+        self.assertEquals(self.counter, 2)
+
+        controlDeferred.callback(secondUnique)
+        self.assertEquals(self.result, secondUnique)
+        self.failUnless(lock.locked)
+        self.assertEquals(self.counter, 3)
+
+        lock.release()
+        self.failIf(lock.locked)
+
+    def testSemaphore(self):
+        N = 13
+        sem = defer.DeferredSemaphore(N)
+
+        controlDeferred = defer.Deferred()
+        def helper(self, arg):
+            self.arg = arg
+            return controlDeferred
+
+        results = []
+        uniqueObject = object()
+        resultDeferred = sem.run(helper, self=self, arg=uniqueObject)
+        resultDeferred.addCallback(results.append)
+        resultDeferred.addCallback(self._incr)
+        self.assertEquals(results, [])
+        self.assertEquals(self.arg, uniqueObject)
+        controlDeferred.callback(None)
+        self.assertEquals(results.pop(), None)
+        self.assertEquals(self.counter, 1)
+
+        self.counter = 0
+        for i in range(1, 1 + N):
+            sem.acquire().addCallback(self._incr)
+            self.assertEquals(self.counter, i)
+
+        sem.acquire().addCallback(self._incr)
+        self.assertEquals(self.counter, N)
+
+        sem.release()
+        self.assertEquals(self.counter, N + 1)
+
+        for i in range(1, 1 + N):
+            sem.release()
+            self.assertEquals(self.counter, N + 1)
+
+    def testQueue(self):
+        N, M = 2, 2
+        queue = defer.DeferredQueue(N, M)
+
+        gotten = []
+
+        for i in range(M):
+            queue.get().addCallback(gotten.append)
+        self.assertRaises(defer.QueueUnderflow, queue.get)
+
+        for i in range(M):
+            queue.put(i)
+            self.assertEquals(gotten, range(i + 1))
+        for i in range(N):
+            queue.put(N + i)
+            self.assertEquals(gotten, range(M))
+        self.assertRaises(defer.QueueOverflow, queue.put, None)
+
+        gotten = []
+        for i in range(N):
+            queue.get().addCallback(gotten.append)
+            self.assertEquals(gotten, range(N, N + i + 1))
+
+        queue = defer.DeferredQueue()
+        gotten = []
+        for i in range(N):
+            queue.get().addCallback(gotten.append)
+        for i in range(N):
+            queue.put(i)
+        self.assertEquals(gotten, range(N))
+        
+        queue = defer.DeferredQueue(size=0)
+        self.assertRaises(defer.QueueOverflow, queue.put, None)
+
+        queue = defer.DeferredQueue(backlog=0)
+        self.assertRaises(defer.QueueUnderflow, queue.get)
+
